import argparse
import logging
import sys

from exobrain.actions.client.accessor import ActionsAccessor
from exobrain.actions.client.http_client import ClientType, create_client

logger = logging.getLogger(__name__)

BASE_URL = "http://127.0.0.1:5001"


def run_demo(base_url: str = BASE_URL) -> None:
    # Create an HTTP client instance for the Exobrain Actions API
    client = create_client(ClientType.REQUESTS, base_url=base_url, timeout=10, verify=False)

    # Create an accessor instance to interact with the Exobrain Actions API
    accessor = ActionsAccessor(client)

    # Does the service respond to a ping?
    accessor.ping()
    logger.info("Ping successful: Exobrain Actions API is alive.")

    # Display the version status
    status = accessor.version_status()
    logger.info("Version Status: %s", status.status)
    logger.info("The service '%s' v%s is running on %s", status.service, status.version, base_url)
    logger.info("Git information: %s", status.git)

    # Display action data
    for service in (
        "alternate-supplier",
        "express-delivery",
        "full-container-deployment",
        "full-truck-load",
        "partial-delivery",
        "stock-rebalancing",
        "substitution",
        "agent-ai-procurement",
    ):
        data = accessor.get_action_data(service=service)
        logger.info("Action Data: %s", data.model_dump_json(indent=2, by_alias=True))

        # Example of calculating an action
        org_id = "5a8c76d9-eee0-4b74-9811-b427f272f8f9"
        action_id = "276728f4-3099-4ff5-b083-748478d15299"
        accessor.calculate(service=service, org_id=org_id, action_id=action_id)
        logger.info(
            "Calculation triggered for service '%s' with org_id '%s' and action_id '%s'.",
            service,
            org_id,
            action_id,
        )

        # Example of updating actuals
        kpis = {"RESPONSIVENESS": 0.85, "FLEXIBILITY": 0.72}
        accessor.update_actuals(service=service, org_id=org_id, action_id=action_id, kpis=kpis)
        logger.info(
            "Actuals updated for service '%s' with org_id '%s' and action_id '%s'.",
            service,
            org_id,
            action_id,
        )


def main() -> None:
    logging.basicConfig(level=logging.INFO, format="%(message)s", stream=sys.stdout)
    parser = argparse.ArgumentParser(description="Exobrain Actions API Demo Client")
    parser.add_argument(
        "--url",
        default=BASE_URL,
        help="Base URL for the Exobrain Actions API (default: %(default)s)",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        help="Enable verbose output",
    )

    args = parser.parse_args()

    if args.verbose:
        logger.info("Connecting to %s", args.url)

    run_demo(base_url=args.url)


if __name__ == "__main__":
    main()



from unittest.mock import Mock

from exobrain.actions.client.accessor import ActionsAccessor
from exobrain.actions.client.http_client.interfaces import HTTPClientFacade, HTTPResponse
from exobrain.actions.schemas.action_data import ActionData

ACTION_DATA = {
    "actionSettings": {
        "V_CO2": {
            "choice": True,
            "defaultValue": "M_SQUARE_PER_M_CUBE",
            "name": "VOLUME_CO2",
            "required": None,
            "type": None,
        }
    },
    "dataNeeded": {
        "CONTEXT": {
            "ITEM_CODE": {"TYPE": "STRING"},
            "LOCATION_CODE": {"TYPE": "STRING"},
            "QUANTITY_AT_RISK": {"TYPE": "STRING"},
        },
        "INPUTS": {
            "ITEM_MASTER": {
                "CATEGORY": "MASTER",
                "ITEM_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "ITEM_DESCRIPTION": {"IS_TAGGABLE": True, "REQUIRED": False, "TYPE": "STRING"},
                "ITEM_TYPE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
                "UNIT_PER_PALLET": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "WEIGHT_PER_UNIT_IN_KG": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_W_KG"},
            },
            "LOCATION_ITEM_MASTER": {
                "CATEGORY": "MASTER",
                "DAYS_OF_SUPPLY_THRESHOLD": {"IS_TAGGABLE": True, "REQUIRED": False, "TYPE": "NUMBER_T_DAYS"},
                "ITEM_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "STANDARD_COST_PER_UNIT": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "TYPE": "OBJECT",
            },
            "LOCATION_MASTERS": {
                "CATEGORY": "MASTER",
                "LOCATION_CITY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_CONTACT_NAME": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_COUNTRY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_DESCRIPTION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_ID": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_PRIMARY_LANGUAGE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_TYPE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_ZIP_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "ARRAY",
            },
            "PROCUREMENT_INTERNAL_ROUTES": {
                "CATEGORY": "ACTION_CONTEXT_VIEW",
                "CO2_EMISSION_PER_KG_SHIPPED": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_CO2_PER_KG"},
                "COST_PER_UNIT_CURRENCY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "DISTANCE_KM": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_D_KM"},
                "LEAD_TIME_DAYS": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_T_DAYS"},
                "MODE_OF_TRANSPORTATION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "OTIF_PERCENT": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "RECEIVING_LOCATION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "ROUTE_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "ROUTE_DESCRIPTION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SHIPPING_LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
            },
            "PROCUREMENT_PO_CONDITION": {
                "CATEGORY": "ACTION_CONTEXT_VIEW",
                "CO2_EMISSION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_V_CO2"},
                "CONDITION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "CONDITION_DESCRIPTION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "COST_PER_UNIT_CURRENCY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "LEAD_TIME_DAYS": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_T_DAYS"},
                "MODE_OF_TRANSPORTATION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "RECEIVING_LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SUPPLIER_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
            },
            "PROCUREMENT_PURCHASE_ORDER": {
                "CATEGORY": "ACTION_CONTEXT_VIEW",
                "CREATION_DATE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "DATE"},
                "DELIVERY_DATE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "DATE"},
                "ITEM_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LINE_NUMBER": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LINE_VALUE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "NUMBER": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "PURCHASE_COST_PER_UNIT": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "QUANTITY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "RECEIVING_LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SHIPPING_DATE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "DATE"},
                "STANDARD_INBOUND_ROUTE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "STATUS": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SUPPLIER_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
                "UNIT_OF_MEASURE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
            },
            "PROCUREMENT_SHIPPING_CONDITIONS": {
                "CATEGORY": "ACTION_CONTEXT_VIEW",
                "CO2_EMISSION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_V_CO2"},
                "CONDITION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "CONDITION_DESCRIPTION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "COST_PER_UNIT_CURRENCY": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "LEAD_TIME_DAYS": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER_T_DAYS"},
                "MODE_OF_TRANSPORTATION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "OTIF_PERCENT": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "RECEIVING_LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SUPPLIER_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
            },
            "PROCUREMENT_SUBSTITUTION": {
                "ALTERNATE_COST_PER_UNIT": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "AMOUNT"},
                "ALTERNATE_SUPPLIER_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "ALTERNATE_SUPPLIER_COUNTRY_OF_ORIGIN": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "CATEGORY": "ACTION_CONTEXT_VIEW",
                "ITEM_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SUBSTITUTE_ITEM_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SUBSTITUTION_QUANTITY_RATIO": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "SUPPLIER_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "ARRAY",
                "UNIT_OF_MEASURE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
            },
        },
        "OUTPUTS": {
            "AGENT_AI_ACTION_DETAILS": {
                "ACTION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "CATEGORY": "ACTION_DETAILS",
                "EXPLANATION": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "SCORE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "TYPE": "OBJECT",
            },
            "AGENT_AI_IMPACT": {
                "CATEGORY": "RISK_IMPACTS",
                "DELIVERY_DATE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "DATE"},
                "LOCATION_CODE": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "STRING"},
                "QUANTITY_AT_RISK": {"IS_TAGGABLE": True, "REQUIRED": True, "TYPE": "NUMBER"},
                "TYPE": "OBJECT",
            },
        },
    },
    "kpis": ["FLEXIBILITY", "RELIABILITY", "RESPONSIVENESS"],
    "name": "AGENT_AI_PROCUREMENT",
    "reasons": [
        "f218957f-65a1-49c5-954a-4b1028c495ab",
        "83b0f297-2c44-4bd1-a2cf-2a051fe53fc7",
        "74972e98-f150-48a1-9f9a-030ca5d192d8",
    ],
}


class TestActionsAccessor:
    def test_version_status(self) -> None:
        version_status = {
            "status": "ok",
            "service": "exobrain-actions",
            "version": "1.2.3",
            "git": {
                "commit": "abc123",
                "branch": "main",
                "date": "2023-10-01T12:27:00Z",
            },
        }
        mock_response = Mock(spec=HTTPResponse, json=Mock(return_value=version_status), raise_for_status=Mock())
        mock_client = Mock(spec=HTTPClientFacade, get=Mock(return_value=mock_response))
        accessor = ActionsAccessor(client=mock_client)
        actual = accessor.version_status()
        assert actual.status == "ok"
        assert actual.service == "exobrain-actions"
        assert actual.version == "1.2.3"
        assert actual.git.commit == "abc123"
        assert actual.git.branch == "main"
        assert actual.git.date == "2023-10-01T12:27:00Z"
        assert mock_client.get.call_count == 1
        assert mock_client.get.call_args[0][0] == "/"

    def test_ping(self) -> None:
        mock_response = Mock(spec=HTTPResponse, raise_for_status=Mock())
        mock_client = Mock(spec=HTTPClientFacade, get=Mock(return_value=mock_response))
        accessor = ActionsAccessor(client=mock_client)
        accessor.ping()
        assert mock_client.get.call_count == 1
        assert mock_client.get.call_args[0][0] == "/health/ping"

    def test_get_action_data(self) -> None:
        mock_response = Mock(spec=HTTPResponse, json=Mock(return_value=ACTION_DATA), raise_for_status=Mock())
        mock_client = Mock(spec=HTTPClientFacade, get=Mock(return_value=mock_response))
        accessor = ActionsAccessor(client=mock_client)
        actual = accessor.get_action_data(service="agent-ai-procurement")
        assert actual == ActionData.model_validate(ACTION_DATA)
        assert mock_client.get.call_count == 1
        assert mock_client.get.call_args[0][0] == "/actions/data"

    def test_calculate(self) -> None:
        mock_response = Mock(spec=HTTPResponse, raise_for_status=Mock())
        mock_client = Mock(spec=HTTPClientFacade, get=Mock(return_value=mock_response))
        accessor = ActionsAccessor(client=mock_client)
        accessor.calculate(service="alternate-supplier", org_id="1234567890", action_id="1234567890")
        assert mock_client.get.call_count == 1
        assert mock_client.get.call_args[0][0] == "/alternate-supplier/organizations/1234567890/actions/1234567890"

    def test_update_actuals(self) -> None:
        mock_response = Mock(spec=HTTPResponse, raise_for_status=Mock())
        mock_client = Mock(spec=HTTPClientFacade, post=Mock(return_value=mock_response))
        accessor = ActionsAccessor(client=mock_client)
        accessor.update_actuals(
            service="alternate-supplier",
            org_id="1234567890",
            action_id="1234567890",
            kpis={"RESPONSIVENESS": 0.85, "FLEXIBILITY": 0.72},
        )
        assert mock_client.post.call_count == 1
        assert (
            mock_client.post.call_args[0][0]
            == "/alternate-supplier/organizations/1234567890/actions/1234567890/actuals"
        )
        assert mock_client.post.call_args[1]["json"] == {"FLEXIBILITY": 0.72, "RESPONSIVENESS": 0.85}



"""
exobrain-actions-client

This package provides a REST API client for interacting with the Exobrain Actions API.

Key features:

- Access to various API resources such as version status and health checks (ping).
- Retrieval, triggering, or updating of action data specific to a service or organization.
- Standardized HTTP calls to different API endpoints (version, health, action data, calculation triggers, KPI updates).
"""

__version__ = "0.1.1"
__release__ = "2025-07-15"
__author__ = "Laurent LAPORTE <laurent.laporte.pro@gmail.com>"
__copyright__ = f"Copyright 2025 {__author__}"



"""
Requests client adapter implementation
"""
# mypy: ignore-errors

import typing as t

import requests
from exobrain.actions.client.http_client.interfaces import (
    HTTPClientFacade,
    HTTPRequest,
    HTTPResponse,
)

__all__ = ["RequestsRequestAdapter", "RequestsResponseAdapter", "RequestsClientAdapter"]


class RequestsRequestAdapter(HTTPRequest):
    """Requests request wrapper"""

    def __init__(self, request: requests.PreparedRequest):
        self._request = request

    @property
    def method(self) -> str:
        return self._request.method or ""

    @property
    def url(self) -> str:
        return self._request.url or ""

    @property
    def headers(self) -> t.Mapping[str, str]:
        return dict(self._request.headers or {})

    @property
    def content(self) -> bytes | None:
        return t.cast(bytes | None, self._request.body)


class RequestsResponseAdapter(HTTPResponse):
    """Requests response wrapper"""

    def __init__(self, response: requests.Response):
        self._response = response

    @property
    def request(self) -> RequestsRequestAdapter:
        return RequestsRequestAdapter(self._response.request)

    @property
    def status_code(self) -> int:
        return self._response.status_code

    @property
    def headers(self) -> t.Mapping[str, str]:
        return dict(self._response.headers)

    @property
    def content(self) -> bytes:
        return self._response.content

    @property
    def text(self) -> str:
        return self._response.text

    def json(self) -> t.Any:
        return self._response.json()


class RequestsClientAdapter(HTTPClientFacade):
    """Requests session client implementation"""

    client: requests.Session

    def __init__(
        self,
        client: requests.Session,
        *,
        base_url: str = "",
        **kwargs: t.Any,
    ):
        super().__init__(client)
        self._base_url = base_url
        self._kwargs = kwargs

    def _build_url(self, url: str) -> str:
        if not self._base_url or url.startswith(("http://", "https://")):
            return url
        return f"{self._base_url.rstrip('/')}/{url.lstrip('/')}"

    def get(self, url: str, **kwargs: t.Any) -> RequestsResponseAdapter:
        kwargs.update({**self._kwargs, **kwargs})
        response = self.client.get(self._build_url(url), **kwargs)
        return RequestsResponseAdapter(response)

    def post(self, url: str, **kwargs: t.Any) -> RequestsResponseAdapter:
        kwargs.update({**self._kwargs, **kwargs})
        response = self.client.post(self._build_url(url), **kwargs)
        return RequestsResponseAdapter(response)

    def put(self, url: str, **kwargs: t.Any) -> RequestsResponseAdapter:
        kwargs.update({**self._kwargs, **kwargs})
        response = self.client.put(self._build_url(url), **kwargs)
        return RequestsResponseAdapter(response)

    def patch(self, url: str, **kwargs: t.Any) -> RequestsResponseAdapter:
        kwargs.update({**self._kwargs, **kwargs})
        response = self.client.patch(self._build_url(url), **kwargs)
        return RequestsResponseAdapter(response)

    def delete(self, url: str, **kwargs: t.Any) -> RequestsResponseAdapter:
        kwargs.update({**self._kwargs, **kwargs})
        response = self.client.delete(self._build_url(url), **kwargs)
        return RequestsResponseAdapter(response)



"""
Flask test client adapter implementation
"""
# mypy: ignore-errors

import typing as t

from exobrain.actions.client.http_client.interfaces import (
    HTTPClientFacade,
    HTTPRequest,
    HTTPResponse,
)
from flask.testing import FlaskClient
from werkzeug.test import TestResponse

__all__ = ["FlaskRequestAdapter", "FlaskResponseAdapter", "FlaskClientAdapter"]


class FlaskRequestAdapter(HTTPRequest):
    """Flask request wrapper (limited support)"""

    def __init__(
        self,
        method: str,
        url: str,
        headers: t.Mapping[str, str] | None = None,
        content: bytes | None = None,
    ):
        self._method = method
        self._url = url
        self._headers = headers or {}
        self._content = content

    @property
    def method(self) -> str:
        return self._method

    @property
    def url(self) -> str:
        return self._url

    @property
    def headers(self) -> t.Mapping[str, str]:
        return self._headers

    @property
    def content(self) -> bytes | None:
        return self._content


class FlaskResponseAdapter(HTTPResponse):
    """Flask test client response wrapper"""

    def __init__(self, response: TestResponse, method: str = "GET"):
        self._response = response
        self._method = method

    @property
    def request(self) -> FlaskRequestAdapter:
        # Flask's test client does not provide a direct request object,
        # so we create a minimal request representation.
        return FlaskRequestAdapter(
            method=self._method,
            url=self._response.request.path,
            headers=dict(self._response.headers),
            content=self._response.get_data(),
        )

    @property
    def status_code(self) -> int:
        return self._response.status_code

    @property
    def headers(self) -> t.Mapping[str, str]:
        return dict(self._response.headers)

    @property
    def content(self) -> bytes:
        return self._response.get_data()

    @property
    def text(self) -> str:
        return self._response.get_data(as_text=True)

    def json(self) -> t.Any:
        return self._response.get_json()


class FlaskClientAdapter(HTTPClientFacade):
    """Flask test client implementation"""

    client: FlaskClient

    def __init__(self, client: FlaskClient):
        super().__init__(client)

    def get(self, url: str, **kwargs: t.Any) -> FlaskResponseAdapter:
        response = self.client.get(url, **kwargs)
        return FlaskResponseAdapter(response, method="get")

    def post(self, url: str, **kwargs: t.Any) -> FlaskResponseAdapter:
        response = self.client.post(url, **kwargs)
        return FlaskResponseAdapter(response, method="post")

    def put(self, url: str, **kwargs: t.Any) -> FlaskResponseAdapter:
        response = self.client.put(url, **kwargs)
        return FlaskResponseAdapter(response, method="put")

    def patch(self, url: str, **kwargs: t.Any) -> FlaskResponseAdapter:
        response = self.client.patch(url, **kwargs)
        return FlaskResponseAdapter(response, method="patch")

    def delete(self, url: str, **kwargs: t.Any) -> FlaskResponseAdapter:
        response = self.client.delete(url, **kwargs)
        return FlaskResponseAdapter(response, method="delete")



"""
HTTP client factory module for creating different types of HTTP clients (HTTPX, Requests, Flask).

Example:
    Creating a requests client with custom configuration:

    >>> from exobrain.actions.client.http_client import ClientType, create_client
    >>> client = create_client(
    ...     ClientType.REQUESTS,
    ...     base_url="https://api.example.com",
    ...     timeout=30,
    ...     verify=True,
    ... )
    >>> response = client.get("/users")
    >>> users = response.json()

    Creating a Flask test client:

    >>> from flask import Flask
    >>> app = Flask(__name__)
    >>> client = create_client(ClientType.FLASK, app=app)
    >>> response = client.get("/health")
    >>> assert response.status_code == 200
"""

# mypy: ignore-errors
import enum
import typing as t

from exobrain.actions.client.http_client.exceptions import HTTPClientError

if t.TYPE_CHECKING:
    from exobrain.actions.client.http_client.interfaces import HTTPClientFacade


class ClientType(enum.StrEnum):
    HTTPX = "httpx"
    REQUESTS = "requests"
    FLASK = "flask"


class ClientCreationError(HTTPClientError):
    """
    Exception raised during the creation of an HTTP client adapter.
    """


class UnsupportedClientTypeError(ClientCreationError):
    """
    Exception raised during the creation of a client when the client type is not supported.
    """

    def __init__(self, client_type: ClientType) -> None:
        message = f"Unsupported client type: {client_type}"
        super().__init__(message)


class MissingArgumentError(ClientCreationError):
    """
    Exception raised when a required argument is missing for client creation.
    """

    def __init__(self) -> None:
        message = "Flask client requires either 'app' or 'client' argument"
        super().__init__(message)


def create_client(client_type: ClientType, **kwargs: t.Any) -> "HTTPClientFacade":
    """
    Create an HTTP client with its adapter based on client type and configuration.

    Args:
        client_type: Type of client to create (HTTPX, REQUESTS or FLASK)
        **kwargs: Client configuration parameters (base_url, verify, timeout, app, etc.)

    Returns:
        Configured HTTP client adapter

    Raises:
        UnsupportedClientTypeError: When client_type is not supported.
        ImportError: When required dependencies are not installed.
        MissingArgumentError: When 'app' or 'client' arguments are not provided for Flask's client.
        TypeError: When the provided 'app' argument is not an instance of Flask's class.
    """
    if client_type == ClientType.HTTPX:
        import httpx
        from exobrain.actions.client.http_client.httpx_adapter import HTTPXClientAdapter

        httpx_client = httpx.Client(**kwargs)
        return HTTPXClientAdapter(httpx_client)

    if client_type == ClientType.REQUESTS:
        import requests
        from exobrain.actions.client.http_client.requests_adapter import RequestsClientAdapter

        session = requests.Session()
        return RequestsClientAdapter(session, **kwargs)

    if client_type == ClientType.FLASK:
        import flask
        from exobrain.actions.client.http_client.flask_adapter import FlaskClientAdapter

        if "app" in kwargs:
            app = kwargs["app"]
            if not isinstance(app, flask.Flask):
                raise TypeError(repr(type(app)))
            flask_client = app.test_client(**kwargs)
        elif "client" in kwargs:
            flask_client = kwargs["client"]
        else:
            raise MissingArgumentError

        return FlaskClientAdapter(flask_client)

    raise UnsupportedClientTypeError(client_type)



"""
Core interfaces for HTTP client facade
"""

import typing as t
from abc import ABC, abstractmethod

from exobrain.actions.client.http_client.exceptions import HTTPClientError

__all__ = ["HTTPClientFacade", "HTTPRequest", "HTTPResponse", "HTTPStatusError"]


STATUS_CLIENT_ERROR = 400
STATUS_ERROR = 600


class HTTPStatusError(HTTPClientError):
    """
    The response had an error HTTP status of 4xx or 5xx.
    """

    def __init__(self, message: str, *, request: "HTTPRequest", response: "HTTPResponse") -> None:
        super().__init__(message)
        self.request = request
        self.response = response


class HTTPRequest(ABC):
    """Common interface for HTTP request objects"""

    @property
    @abstractmethod
    def method(self) -> str:
        """HTTP method"""

    @property
    @abstractmethod
    def url(self) -> str:
        """Request URL"""

    @property
    @abstractmethod
    def headers(self) -> t.Mapping[str, str]:
        """Request headers"""

    @property
    @abstractmethod
    def content(self) -> bytes | None:
        """Request body content"""


class HTTPResponse(ABC):
    """Common interface for HTTP response objects"""

    @property
    @abstractmethod
    def request(self) -> HTTPRequest:
        """The request that generated this response"""

    @property
    @abstractmethod
    def status_code(self) -> int:
        """HTTP status code"""

    @property
    @abstractmethod
    def headers(self) -> t.Mapping[str, str]:
        """Response headers"""

    @property
    @abstractmethod
    def content(self) -> bytes:
        """Response body as bytes"""

    @property
    @abstractmethod
    def text(self) -> str:
        """Response body as text"""

    @abstractmethod
    def json(self) -> t.Any:
        """Parse response body as JSON"""

    def raise_for_status(self) -> None:
        """
        Check the response status code and raises an exception if it's an error.

        Raises:
            HTTPStatusError: If the status code indicates a client or server error (4xx or 5xx).
        """
        import json

        if STATUS_CLIENT_ERROR <= self.status_code < STATUS_ERROR:
            error_message = f"HTTP Error {self.status_code} for url: {self.request.url}"

            try:
                error_detail = self.json()
                error_message += f"\nResponse JSON: {json.dumps(error_detail, indent=2)}"
            except (json.JSONDecodeError, ValueError):
                error_message += f"\nResponse Text: {self.text}"

            raise HTTPStatusError(error_message, request=self.request, response=self)


class HTTPClientFacade(ABC):
    """Common interface for HTTP clients with context manager support"""

    def __init__(self, client: t.Any):
        self.client = client

    @abstractmethod
    def get(
        self,
        url: str,
        *,
        params: t.Mapping[str, t.Any] | None = None,
        headers: t.Mapping[str, str] | None = None,
        cookies: t.Mapping[str, str] | None = None,
        follow_redirects: bool = True,
        timeout: float | None = None,
    ) -> HTTPResponse:
        """Perform a GET request to the specified URL."""

    @abstractmethod
    def post(
        self,
        url: str,
        *,
        data: t.Any = None,
        json: t.Any = None,
        params: t.Mapping[str, t.Any] | None = None,
        headers: t.Mapping[str, str] | None = None,
        cookies: t.Mapping[str, str] | None = None,
        files: t.Mapping[str, t.Any] | None = None,
        follow_redirects: bool = True,
        timeout: float | None = None,
    ) -> HTTPResponse:
        """Perform a POST request to the specified URL."""

    @abstractmethod
    def put(
        self,
        url: str,
        *,
        data: t.Any = None,
        json: t.Any = None,
        params: t.Mapping[str, t.Any] | None = None,
        headers: t.Mapping[str, str] | None = None,
        cookies: t.Mapping[str, str] | None = None,
        files: t.Mapping[str, t.Any] | None = None,
        follow_redirects: bool = True,
        timeout: float | None = None,
    ) -> HTTPResponse:
        """Perform a PUT request to the specified URL."""

    @abstractmethod
    def patch(
        self,
        url: str,
        *,
        data: t.Any = None,
        json: t.Any = None,
        params: t.Mapping[str, t.Any] | None = None,
        headers: t.Mapping[str, str] | None = None,
        cookies: t.Mapping[str, str] | None = None,
        files: t.Mapping[str, t.Any] | None = None,
        follow_redirects: bool = True,
        timeout: float | None = None,
    ) -> HTTPResponse:
        """Perform a PATCH request to the specified URL."""

    @abstractmethod
    def delete(
        self,
        url: str,
        *,
        params: t.Mapping[str, t.Any] | None = None,
        headers: t.Mapping[str, str] | None = None,
        cookies: t.Mapping[str, str] | None = None,
        follow_redirects: bool = True,
        timeout: float | None = None,
    ) -> HTTPResponse:
        """Perform a DELETE request to the specified URL."""



"""
HTTPX client adapter implementation
"""
# mypy: ignore-errors

import typing as t

import httpx
from exobrain.actions.client.http_client.interfaces import (
    HTTPClientFacade,
    HTTPRequest,
    HTTPResponse,
)

__all__ = ["HTTPXRequestAdapter", "HTTPXResponseAdapter", "HTTPXClientAdapter"]


class HTTPXRequestAdapter(HTTPRequest):
    """HTTPX request wrapper"""

    def __init__(self, request: httpx.Request):
        self._request = request

    @property
    def method(self) -> str:
        return self._request.method

    @property
    def url(self) -> str:
        return str(self._request.url)

    @property
    def headers(self) -> t.Mapping[str, str]:
        return dict(self._request.headers)

    @property
    def content(self) -> bytes | None:
        return self._request.content


class HTTPXResponseAdapter(HTTPResponse):
    """HTTPX response wrapper"""

    def __init__(self, response: httpx.Response):
        self._response = response

    @property
    def request(self) -> HTTPXRequestAdapter:
        return HTTPXRequestAdapter(self._response.request)

    @property
    def status_code(self) -> int:
        return self._response.status_code

    @property
    def headers(self) -> t.Mapping[str, str]:
        return dict(self._response.headers)

    @property
    def content(self) -> bytes:
        return self._response.content

    @property
    def text(self) -> str:
        return self._response.text

    def json(self) -> t.Any:
        return self._response.json()


class HTTPXClientAdapter(HTTPClientFacade):
    """HTTPX client implementation"""

    client: httpx.Client

    def __init__(self, client: httpx.Client):
        super().__init__(client)

    def get(self, url: str, **kwargs: t.Any) -> HTTPXResponseAdapter:
        response = self.client.get(url, **kwargs)
        return HTTPXResponseAdapter(response)

    def post(self, url: str, **kwargs: t.Any) -> HTTPXResponseAdapter:
        response = self.client.post(url, **kwargs)
        return HTTPXResponseAdapter(response)

    def put(self, url: str, **kwargs: t.Any) -> HTTPXResponseAdapter:
        response = self.client.put(url, **kwargs)
        return HTTPXResponseAdapter(response)

    def patch(self, url: str, **kwargs: t.Any) -> HTTPXResponseAdapter:
        response = self.client.patch(url, **kwargs)
        return HTTPXResponseAdapter(response)

    def delete(self, url: str, **kwargs: t.Any) -> HTTPXResponseAdapter:
        response = self.client.delete(url, **kwargs)
        return HTTPXResponseAdapter(response)



import typing as t
from uuid import UUID

from exobrain.actions.client.http_client.interfaces import HTTPClientFacade
from exobrain.actions.schemas.action_data import ActionData
from exobrain.actions.schemas.version_info import VersionStatus


class ActionsAccessor:
    """
    Base class for accessing resources in the Exobrain Actions API.
    """

    def __init__(self, client: HTTPClientFacade) -> None:
        """
        Initialize the BaseAccessor with a client instance.

        :param client: An instance of the Client class.
        """
        self.client = client

    def version_status(self) -> VersionStatus:
        """
        Retrieve the version status of the Exobrain Actions API.

        :return: An instance of VersionStatus containing version details.
        """
        response = self.client.get("/")
        response.raise_for_status()
        json_data = response.json()
        return VersionStatus.model_validate(json_data)

    def ping(self) -> None:
        """
        Ping the Exobrain Actions API to check if it is alive.

        :raises HTTPStatusError: If the ping fails.
        """
        response = self.client.get("/health/ping")
        response.raise_for_status()

    def get_action_data(self, service: str = "") -> ActionData:
        """
        Retrieve action data for a specific service or the currently running service.

        Args:
            service: The service name (e.g., "alternate-supplier").
            If not provided, defaults to the currently running service.

        Returns:
            An instance of ActionData containing the action data.
        """
        params = {"service": service} if service else {}
        response = self.client.get("/actions/data", params=params)
        response.raise_for_status()
        return ActionData.model_validate(response.json())

    def calculate(self, service: str, org_id: str | UUID, action_id: str | UUID) -> None:
        """
        Trigger the calculation for a specific action.

        Args:
            service: The service name (e.g., "alternate-supplier").
            org_id: The organization ID.
            action_id: The action ID.
        """
        url = f"/{service}/organizations/{org_id}/actions/{action_id}"
        response = self.client.get(url)
        response.raise_for_status()

    def update_actuals(
        self,
        service: str,
        org_id: str | UUID,
        action_id: str | UUID,
        kpis: t.Mapping[str, float],
    ) -> None:
        """
        Update the KPIs for a specific action.

        Args:
            service: The service name (e.g., "alternate-supplier").
            org_id: The organization ID.
            action_id: The action ID.
            kpis: A dictionary containing the KPI data to update.
        """
        url = f"/{service}/organizations/{org_id}/actions/{action_id}/actuals"
        response = self.client.post(url, json=kpis)
        response.raise_for_status()
